@@ -28,7 +28,6 @@
 #include <thunar/thunar-details-view-ui.h>
 #include <thunar/thunar-gtk-extensions.h>
 #include <thunar/thunar-private.h>
-#include <thunar/thunar-text-renderer.h>
 #include <thunar/thunar-preferences.h>
 
 
@@ -205,10 +204,12 @@ thunar_details_view_init (ThunarDetailsView *details_view)
    */
   g_signal_connect (G_OBJECT (details_view), "notify::zoom-level", G_CALLBACK (thunar_details_view_zoom_level_changed), NULL);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   /* setup the details view actions */
   gtk_action_group_add_actions (THUNAR_STANDARD_VIEW (details_view)->action_group,
                                 action_entries, G_N_ELEMENTS (action_entries),
                                 GTK_WIDGET (details_view));
+G_GNUC_END_IGNORE_DEPRECATIONS
 
   /* create the tree view to embed */
   tree_view = exo_tree_view_new ();
@@ -224,7 +225,6 @@ thunar_details_view_init (ThunarDetailsView *details_view)
   gtk_widget_show (tree_view);
 
   /* configure general aspects of the details view */
-  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), TRUE);
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tree_view), TRUE);
 
   /* enable rubberbanding (if supported) */
@@ -237,11 +237,11 @@ thunar_details_view_init (ThunarDetailsView *details_view)
                           G_CALLBACK (thunar_details_view_row_changed), details_view);
 
   /* allocate the shared right-aligned text renderer */
-  right_aligned_renderer = g_object_new (THUNAR_TYPE_TEXT_RENDERER, "xalign", 1.0f, NULL);
+  right_aligned_renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT, "xalign", 1.0f, NULL);
   g_object_ref_sink (G_OBJECT (right_aligned_renderer));
 
   /* allocate the shared left-aligned text renderer */
-  left_aligned_renderer = g_object_new (THUNAR_TYPE_TEXT_RENDERER, "xalign", 0.0f, NULL);
+  left_aligned_renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT, "xalign", 0.0f, NULL);
   g_object_ref_sink (G_OBJECT (left_aligned_renderer));
 
   /* allocate the tree view columns */
@@ -268,7 +268,8 @@ thunar_details_view_init (ThunarDetailsView *details_view)
                                                NULL);
 
           /* add the name renderer */
-          g_object_set (G_OBJECT (THUNAR_STANDARD_VIEW (details_view)->name_renderer), "xalign", 0.0, NULL);
+          g_object_set (G_OBJECT (THUNAR_STANDARD_VIEW (details_view)->name_renderer),
+                        "xalign", 0.0, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
           gtk_tree_view_column_pack_start (details_view->columns[column], THUNAR_STANDARD_VIEW (details_view)->name_renderer, TRUE);
           gtk_tree_view_column_set_attributes (details_view->columns[column], THUNAR_STANDARD_VIEW (details_view)->name_renderer,
                                                "text", THUNAR_COLUMN_NAME,
@@ -281,7 +282,7 @@ thunar_details_view_init (ThunarDetailsView *details_view)
       else
         {
           /* size is right aligned, everything else is left aligned */
-          renderer = (column == THUNAR_COLUMN_SIZE) ? right_aligned_renderer : left_aligned_renderer;
+          renderer = (column == THUNAR_COLUMN_SIZE || column == THUNAR_COLUMN_SIZE_IN_BYTES) ? right_aligned_renderer : left_aligned_renderer;
 
           /* add the renderer */
           gtk_tree_view_column_pack_start (details_view->columns[column], renderer, TRUE);
@@ -405,7 +406,6 @@ thunar_details_view_get_accessible (GtkWidget *widget)
 }
 
 
-
 static void
 thunar_details_view_connect_ui_manager (ThunarStandardView *standard_view,
                                         GtkUIManager       *ui_manager)
@@ -413,8 +413,10 @@ thunar_details_view_connect_ui_manager (ThunarStandardView *standard_view,
   ThunarDetailsView *details_view = THUNAR_DETAILS_VIEW (standard_view);
   GError            *error = NULL;
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   details_view->ui_merge_id = gtk_ui_manager_add_ui_from_string (ui_manager, thunar_details_view_ui,
                                                                  thunar_details_view_ui_length, &error);
+G_GNUC_END_IGNORE_DEPRECATIONS
   if (G_UNLIKELY (details_view->ui_merge_id == 0))
     {
       g_error ("Failed to merge ThunarDetailsView menus: %s", error->message);
@@ -428,7 +430,9 @@ static void
 thunar_details_view_disconnect_ui_manager (ThunarStandardView *standard_view,
                                            GtkUIManager       *ui_manager)
 {
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   gtk_ui_manager_remove_ui (ui_manager, THUNAR_DETAILS_VIEW (standard_view)->ui_merge_id);
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 
@@ -440,7 +444,7 @@ thunar_details_view_get_selected_items (ThunarStandardView *standard_view)
 
   _thunar_return_val_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view), NULL);
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
   return gtk_tree_selection_get_selected_rows (selection, NULL);
 }
 
@@ -453,7 +457,7 @@ thunar_details_view_select_all (ThunarStandardView *standard_view)
 
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
   gtk_tree_selection_select_all (selection);
 }
 
@@ -466,7 +470,7 @@ thunar_details_view_unselect_all (ThunarStandardView *standard_view)
 
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
   gtk_tree_selection_unselect_all (selection);
 }
 
@@ -495,7 +499,7 @@ thunar_details_view_selection_invert (ThunarStandardView *standard_view)
 
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
 
   /* block updates */
   g_signal_handlers_block_by_func (selection, thunar_standard_view_selection_changed, standard_view);
@@ -530,7 +534,7 @@ thunar_details_view_select_path (ThunarStandardView *standard_view,
 
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
   gtk_tree_selection_select_path (selection, path);
 }
 
@@ -547,17 +551,17 @@ thunar_details_view_set_cursor (ThunarStandardView *standard_view,
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
   /* make sure the name renderer is editable */
-  mode = standard_view->name_renderer->mode;
-  standard_view->name_renderer->mode = GTK_CELL_RENDERER_MODE_EDITABLE;
+  g_object_get ( G_OBJECT (standard_view->name_renderer), "mode", &mode, NULL);
+  g_object_set ( G_OBJECT (standard_view->name_renderer), "mode", GTK_CELL_RENDERER_MODE_EDITABLE, NULL);
 
   /* tell the tree view to start editing the given row */
-  column = gtk_tree_view_get_column (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), 0);
-  gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (GTK_BIN (standard_view)->child),
+  column = gtk_tree_view_get_column (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), 0);
+  gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))),
                                     path, column, standard_view->name_renderer,
                                     start_editing);
 
   /* reset the name renderer mode */
-  standard_view->name_renderer->mode = mode;
+  g_object_set (G_OBJECT (standard_view->name_renderer), "mode", mode, NULL);
 }
 
 
@@ -574,8 +578,8 @@ thunar_details_view_scroll_to_path (ThunarStandardView *standard_view,
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
   /* tell the tree view to scroll to the given row */
-  column = gtk_tree_view_get_column (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), 0);
-  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), path, column, use_align, row_align, col_align);
+  column = gtk_tree_view_get_column (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), 0);
+  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), path, column, use_align, row_align, col_align);
 }
 
 
@@ -589,7 +593,7 @@ thunar_details_view_get_path_at_pos (ThunarStandardView *standard_view,
 
   _thunar_return_val_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view), NULL);
 
-  if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), x, y, &path, NULL))
+  if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), x, y, &path, NULL))
     return path;
 
   return NULL;
@@ -604,7 +608,7 @@ thunar_details_view_get_visible_range (ThunarStandardView *standard_view,
 {
   _thunar_return_val_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view), FALSE);
 
-  return gtk_tree_view_get_visible_range (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), start_path, end_path);
+  return gtk_tree_view_get_visible_range (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), start_path, end_path);
 }
 
 
@@ -614,7 +618,7 @@ thunar_details_view_highlight_path (ThunarStandardView *standard_view,
                                     GtkTreePath        *path)
 {
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
-  gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), path, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
+  gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), path, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
 }
 
 
@@ -726,7 +730,7 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
       if (path == NULL)
         {
           /* open the context menu */
-          thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view), event->button, event->time);
+          thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view));
         }
       else
         {
@@ -749,7 +753,7 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
                 }
 
               /* show the context menu */
-              thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view), event->button, event->time);
+              thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view));
             }
           gtk_tree_path_free (path);
         }
@@ -783,9 +787,11 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
 
               action = thunar_gtk_ui_manager_get_action_by_name (THUNAR_STANDARD_VIEW (details_view)->ui_manager, action_name);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
               /* emit the action */
               if (G_LIKELY (action != NULL))
                   gtk_action_activate (action);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
               /* release the file reference */
               g_object_unref (G_OBJECT (file));
@@ -809,9 +815,9 @@ thunar_details_view_key_press_event (GtkTreeView       *tree_view,
                                      ThunarDetailsView *details_view)
 {
   /* popup context menu if "Menu" or "<Shift>F10" is pressed */
-  if (event->keyval == GDK_Menu || ((event->state & GDK_SHIFT_MASK) != 0 && event->keyval == GDK_F10))
+  if (event->keyval == GDK_KEY_Menu || ((event->state & GDK_SHIFT_MASK) != 0 && event->keyval == GDK_KEY_F10))
     {
-      thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view), 0, event->time);
+      thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view));
       return TRUE;
     }
 
@@ -836,10 +842,12 @@ thunar_details_view_row_activated (GtkTreeView       *tree_view,
   gtk_tree_selection_unselect_all (selection);
   gtk_tree_selection_select_path (selection, path);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   /* emit the "open" action */
   action = thunar_gtk_ui_manager_get_action_by_name (THUNAR_STANDARD_VIEW (details_view)->ui_manager, "open");
   if (G_LIKELY (action != NULL))
     gtk_action_activate (action);
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 
@@ -880,7 +888,7 @@ thunar_details_view_columns_changed (ThunarColumnModel *column_model,
       /* change the order of the column relative to its predecessor */
       if (G_LIKELY (column > 0))
         {
-          gtk_tree_view_move_column_after (GTK_TREE_VIEW (GTK_BIN (details_view)->child),
+          gtk_tree_view_move_column_after (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (details_view))),
                                            details_view->columns[column_order[column]],
                                            details_view->columns[column_order[column - 1]]);
         }
@@ -911,7 +919,9 @@ thunar_details_view_action_setup_columns (GtkAction         *action,
                                           ThunarDetailsView *details_view)
 {
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (details_view));
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (GTK_IS_ACTION (action));
+G_GNUC_END_IGNORE_DEPRECATIONS
 
   /* popup the column editor dialog */
   thunar_show_column_editor (details_view);
@@ -986,7 +996,7 @@ thunar_details_view_set_fixed_columns (ThunarDetailsView *details_view,
       /* for fixed columns mode, we can enable the fixed height
        * mode to improve the performance of the GtkTreeVeiw.
        */
-      gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW (GTK_BIN (details_view)->child), fixed_columns);
+      gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (details_view))), fixed_columns);
 
       /* notify listeners */
       g_object_notify (G_OBJECT (details_view), "fixed-columns");
