@@ -3,18 +3,18 @@
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
  *
- * This program is free software; you can redistribute it and/or 
+ * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of 
+ * published by the Free Software Foundation; either version 2 of
  * the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public 
- * License along with this program; if not, write to the Free 
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
@@ -34,7 +34,7 @@
 #include <thunar/thunar-icon-factory.h>
 #include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
-#include <thunar/thunar-thumbnail-frame.h>
+#include <thunar/thunar-util.h>
 
 
 
@@ -332,12 +332,12 @@ thunar_icon_factory_sweep_timer (gpointer user_data)
 
   _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), FALSE);
 
-  GDK_THREADS_ENTER ();
+THUNAR_THREADS_ENTER
 
   /* ditch all icons whose ref_count is 1 */
   g_hash_table_foreach_remove (factory->icon_cache, (GHRFunc) thunar_icon_check_sweep, factory);
 
-  GDK_THREADS_LEAVE ();
+THUNAR_THREADS_LEAVE
 
   return FALSE;
 }
@@ -398,6 +398,26 @@ thumbnail_needs_frame (const GdkPixbuf *thumbnail,
 
 
 
+static GdkPixbuf*
+thunar_icon_factory_get_thumbnail_frame (void)
+{
+  GInputStream *stream;
+  static GdkPixbuf *frame = NULL;
+
+  if (G_LIKELY (frame != NULL))
+     return frame;
+
+  stream = g_resources_open_stream ("/org/xfce/thunar/thumbnail-frame.png", 0, NULL);
+  if (G_UNLIKELY (stream != NULL)) {
+    frame = gdk_pixbuf_new_from_stream (stream, NULL, NULL);
+    g_object_unref (stream);
+  }
+
+  return frame;
+}
+
+
+
 static GdkPixbuf*
 thunar_icon_factory_load_from_file (ThunarIconFactory *factory,
                                     const gchar       *path,
@@ -423,8 +443,11 @@ thunar_icon_factory_load_from_file (ThunarIconFactory *factory,
       /* check if we want to add a frame to the image (we really don't
        * want to do this for icons displayed in the details view).
        */
-      needs_frame = (strstr (path, G_DIR_SEPARATOR_S ".thumbnails" G_DIR_SEPARATOR_S) != NULL)
-                 && (size >= 32) && thumbnail_needs_frame (pixbuf, width, height);
+      needs_frame = FALSE;
+      /* Disabled for 1.8.0 release, will be made optional later
+       * needs_frame = (strstr (path, G_DIR_SEPARATOR_S ".cache/thumbnails" G_DIR_SEPARATOR_S) != NULL)
+       *         && (size >= 32) && thumbnail_needs_frame (pixbuf, width, height);
+       */
 
       /* be sure to make framed thumbnails fit into the size */
       if (G_LIKELY (needs_frame))
@@ -451,10 +474,9 @@ thunar_icon_factory_load_from_file (ThunarIconFactory *factory,
       if (G_LIKELY (needs_frame))
         {
           /* add a frame to the thumbnail */
-          frame = gdk_pixbuf_new_from_inline (-1, thunar_thumbnail_frame, FALSE, NULL);
+          frame = thunar_icon_factory_get_thumbnail_frame ();
           tmp = exo_gdk_pixbuf_frame (pixbuf, frame, 4, 3, 5, 6);
           g_object_unref (G_OBJECT (pixbuf));
-          g_object_unref (G_OBJECT (frame));
           pixbuf = tmp;
         }
     }
@@ -494,15 +516,19 @@ thunar_icon_factory_lookup_icon (ThunarIconFactory *factory,
         }
       else
         {
+          /* FIXME: is there a better approach? */
+          if (g_strcmp0 (name, "inode-directory") == 0)
+            name = "folder";
+
           /* check if the icon theme contains an icon of that name */
-          icon_info = gtk_icon_theme_lookup_icon (factory->icon_theme, name, size, 0);
+          icon_info = gtk_icon_theme_lookup_icon (factory->icon_theme, name, size, GTK_ICON_LOOKUP_FORCE_SIZE);
           if (G_LIKELY (icon_info != NULL))
             {
               /* try to load the pixbuf from the icon info */
               pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
 
               /* cleanup */
-              gtk_icon_info_free (icon_info);
+              g_object_unref (icon_info);
             }
         }
 
@@ -533,7 +559,7 @@ thunar_icon_factory_lookup_icon (ThunarIconFactory *factory,
                                                             thunar_icon_factory_sweep_timer_destroy);
     }
 
-  return g_object_ref (G_OBJECT (pixbuf));
+  return GDK_PIXBUF (g_object_ref (G_OBJECT (pixbuf)));
 }
 
 
@@ -665,7 +691,7 @@ thunar_icon_factory_get_for_icon_theme (GtkIconTheme *icon_theme)
     {
       /* allocate a new factory and connect it to the icon theme */
       factory = g_object_new (THUNAR_TYPE_ICON_FACTORY, NULL);
-      factory->icon_theme = g_object_ref (G_OBJECT (icon_theme));
+      factory->icon_theme = GTK_ICON_THEME (g_object_ref (G_OBJECT (icon_theme)));
       g_object_set_qdata (G_OBJECT (factory->icon_theme), thunar_icon_factory_quark, factory);
 
       /* connect the "show-thumbnails" property to the global preference */
@@ -749,7 +775,7 @@ thunar_icon_factory_load_icon (ThunarIconFactory        *factory,
 {
   _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), NULL);
   _thunar_return_val_if_fail (size > 0, NULL);
-  
+
   /* cannot happen unless there's no XSETTINGS manager
    * for the default screen, but just in case...
    */
@@ -833,9 +859,9 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
           if (G_IS_THEMED_ICON (gicon))
             {
               /* we have a themed preview icon, look it up using the icon theme */
-              icon_info = 
-                gtk_icon_theme_lookup_by_gicon (factory->icon_theme, 
-                                                gicon, icon_size, 
+              icon_info =
+                gtk_icon_theme_lookup_by_gicon (factory->icon_theme,
+                                                gicon, icon_size,
                                                 GTK_ICON_LOOKUP_USE_BUILTIN
                                                 | GTK_ICON_LOOKUP_FORCE_SIZE);
 
@@ -844,7 +870,7 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
                 {
                   /* try to load the pixbuf from the icon info */
                   icon = gtk_icon_info_load_icon (icon_info, NULL);
-                  gtk_icon_info_free (icon_info);
+                  g_object_unref (icon_info);
                 }
             }
           else if (G_IS_LOADABLE_ICON (gicon))
