@@ -24,6 +24,7 @@
 
 #include <gio/gio.h>
 
+#include <thunar/thunar-application.h>
 #include <thunar/thunar-enum-types.h>
 #include <thunar/thunar-gio-extensions.h>
 #include <thunar/thunar-io-scan-directory.h>
@@ -32,6 +33,7 @@
 #include <thunar/thunar-job.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-simple-job.h>
+#include <thunar/thunar-thumbnail-cache.h>
 #include <thunar/thunar-transfer-job.h>
 
 
@@ -48,14 +50,14 @@ _tij_collect_nofollow (ThunarJob *job,
   GList  *lp;
 
   /* recursively collect the files */
-  for (lp = base_file_list; 
-       err == NULL && lp != NULL && !exo_job_is_cancelled (EXO_JOB (job)); 
+  for (lp = base_file_list;
+       err == NULL && lp != NULL && !exo_job_is_cancelled (EXO_JOB (job));
        lp = lp->next)
     {
       /* try to scan the directory */
-      child_file_list = thunar_io_scan_directory (job, lp->data, 
-                                                  G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, 
-                                                  TRUE, unlinking, &err);
+      child_file_list = thunar_io_scan_directory (job, lp->data,
+                                                  G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                                  TRUE, unlinking, FALSE, &err);
 
       /* prepend the new files to the existing list */
       file_list = thunar_g_file_list_prepend (file_list, lp->data);
@@ -82,9 +84,9 @@ _tij_collect_nofollow (ThunarJob *job,
 
 
 static gboolean
-_thunar_io_jobs_create (ThunarJob   *job,
-                        GValueArray *param_values,
-                        GError     **error)
+_thunar_io_jobs_create (ThunarJob  *job,
+                        GArray     *param_values,
+                        GError    **error)
 {
   GFileOutputStream *stream;
   ThunarJobResponse  response = THUNAR_JOB_RESPONSE_CANCEL;
@@ -94,21 +96,36 @@ _thunar_io_jobs_create (ThunarJob   *job,
   GList             *lp;
   gchar             *base_name;
   gchar             *display_name;
-  
+  GFile             *template_file;
+  GFileInputStream  *template_stream = NULL;
+
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 1, FALSE);
+  _thunar_return_val_if_fail (param_values->len == 2, FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   /* get the file list */
-  file_list = g_value_get_boxed (g_value_array_get_nth (param_values, 0));
+  file_list = g_value_get_boxed (&g_array_index (param_values, GValue, 0));
+  template_file = g_value_get_object (&g_array_index (param_values, GValue, 1));
 
   /* we know the total amount of files to be processed */
   thunar_job_set_total_files (THUNAR_JOB (job), file_list);
 
+  /* check if we need to open the template */
+  if (template_file != NULL)
+    {
+      /* open read stream to feed in the new files */
+      template_stream = g_file_read (template_file, exo_job_get_cancellable (EXO_JOB (job)), &err);
+      if (G_UNLIKELY (template_stream == NULL))
+        {
+          g_propagate_error (error, err);
+          return FALSE;
+        }
+    }
+
   /* iterate over all files in the list */
-  for (lp = file_list; 
-       err == NULL && lp != NULL && !exo_job_is_cancelled (EXO_JOB (job)); 
+  for (lp = file_list;
+       err == NULL && lp != NULL && !exo_job_is_cancelled (EXO_JOB (job));
        lp = lp->next)
     {
       g_assert (G_IS_FILE (lp->data));
@@ -118,8 +135,8 @@ _thunar_io_jobs_create (ThunarJob   *job,
 
 again:
       /* try to create the file */
-      stream = g_file_create (lp->data, 
-                              G_FILE_CREATE_NONE, 
+      stream = g_file_create (lp->data,
+                              G_FILE_CREATE_NONE,
                               exo_job_get_cancellable (EXO_JOB (job)),
                               &err);
 
@@ -159,8 +176,8 @@ again:
                 }
 
               /* ask the user whether he wants to overwrite the existing file */
-              response = thunar_job_ask_overwrite (THUNAR_JOB (job), 
-                                                   _("The file \"%s\" already exists"), 
+              response = thunar_job_ask_overwrite (THUNAR_JOB (job),
+                                                   _("The file \"%s\" already exists"),
                                                    display_name);
 
               /* check if we should overwrite */
@@ -170,11 +187,11 @@ again:
                   if (g_file_delete (lp->data, exo_job_get_cancellable (EXO_JOB (job)), &err))
                     goto again;
                 }
-              
+
               /* clean up */
               g_free (display_name);
             }
-          else 
+          else
             {
               /* determine display name of the file */
               base_name = g_file_get_basename (lp->data);
@@ -182,7 +199,7 @@ again:
               g_free (base_name);
 
               /* ask the user whether to skip/retry this path (cancels the job if not) */
-              response = thunar_job_ask_skip (THUNAR_JOB (job), 
+              response = thunar_job_ask_skip (THUNAR_JOB (job),
                                               _("Failed to create empty file \"%s\": %s"),
                                               display_name, err->message);
               g_free (display_name);
@@ -195,9 +212,24 @@ again:
             }
         }
       else
-        g_object_unref (stream);
+        {
+          if (template_stream != NULL)
+            {
+              /* write the template into the new file */
+              g_output_stream_splice (G_OUTPUT_STREAM (stream),
+                                      G_INPUT_STREAM (template_stream),
+                                      G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+                                      exo_job_get_cancellable (EXO_JOB (job)),
+                                      NULL);
+            }
+
+          g_object_unref (stream);
+        }
     }
 
+  if (template_stream != NULL)
+    g_object_unref (template_stream);
+
   /* check if we have failed */
   if (err != NULL)
     {
@@ -218,18 +250,20 @@ again:
 
 
 ThunarJob *
-thunar_io_jobs_create_files (GList *file_list)
+thunar_io_jobs_create_files (GList *file_list,
+                             GFile *template_file)
 {
-  return thunar_simple_job_launch (_thunar_io_jobs_create, 1,
-                                   THUNAR_TYPE_G_FILE_LIST, file_list);
+  return thunar_simple_job_launch (_thunar_io_jobs_create, 2,
+                                   THUNAR_TYPE_G_FILE_LIST, file_list,
+                                   G_TYPE_FILE, template_file);
 }
 
 
 
 static gboolean
-_thunar_io_jobs_mkdir (ThunarJob   *job,
-                       GValueArray *param_values,
-                       GError     **error)
+_thunar_io_jobs_mkdir (ThunarJob  *job,
+                       GArray     *param_values,
+                       GError    **error)
 {
   ThunarJobResponse response;
   GFileInfo        *info;
@@ -241,15 +275,15 @@ _thunar_io_jobs_mkdir (ThunarJob   *job,
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 1, FALSE);
+  _thunar_return_val_if_fail (param_values->len == 1, FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  file_list = g_value_get_boxed (g_value_array_get_nth (param_values, 0));
+  file_list = g_value_get_boxed (&g_array_index (param_values, GValue, 0));
 
   /* we know the total list of files to process */
   thunar_job_set_total_files (THUNAR_JOB (job), file_list);
 
-  for (lp = file_list; 
+  for (lp = file_list;
        err == NULL && lp != NULL && !exo_job_is_cancelled (EXO_JOB (job));
        lp = lp->next)
     {
@@ -296,7 +330,7 @@ again:
                 }
 
               /* ask the user whether he wants to overwrite the existing file */
-              response = thunar_job_ask_overwrite (THUNAR_JOB (job), 
+              response = thunar_job_ask_overwrite (THUNAR_JOB (job),
                                                    _("The file \"%s\" already exists"),
                                                    display_name);
 
@@ -319,7 +353,7 @@ again:
               g_free (base_name);
 
               /* ask the user whether to skip/retry this path (cancels the job if not) */
-              response = thunar_job_ask_skip (THUNAR_JOB (job), 
+              response = thunar_job_ask_skip (THUNAR_JOB (job),
                                               _("Failed to create directory \"%s\": %s"),
                                               display_name, err->message);
               g_free (display_name);
@@ -347,7 +381,7 @@ again:
 
   /* emit the "new-files" signal with the given file list */
   thunar_job_new_files (THUNAR_JOB (job), file_list);
-  
+
   return TRUE;
 }
 
@@ -363,25 +397,27 @@ thunar_io_jobs_make_directories (GList *file_list)
 
 
 static gboolean
-_thunar_io_jobs_unlink (ThunarJob   *job,
-                        GValueArray *param_values,
-                        GError     **error)
+_thunar_io_jobs_unlink (ThunarJob  *job,
+                        GArray     *param_values,
+                        GError    **error)
 {
-  ThunarJobResponse response;
-  GFileInfo        *info;
-  GError           *err = NULL;
-  GList            *file_list;
-  GList            *lp;
-  gchar            *base_name;
-  gchar            *display_name;
+  ThunarThumbnailCache *thumbnail_cache;
+  ThunarApplication    *application;
+  ThunarJobResponse     response;
+  GFileInfo            *info;
+  GError               *err = NULL;
+  GList                *file_list;
+  GList                *lp;
+  gchar                *base_name;
+  gchar                *display_name;
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 1, FALSE);
+  _thunar_return_val_if_fail (param_values->len == 1, FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   /* get the file list */
-  file_list = g_value_get_boxed (g_value_array_get_nth (param_values, 0));
+  file_list = g_value_get_boxed (&g_array_index (param_values, GValue, 0));
 
   /* tell the user that we're preparing to unlink the files */
   exo_job_info_message (EXO_JOB (job), _("Preparing..."));
@@ -404,6 +440,11 @@ _thunar_io_jobs_unlink (ThunarJob   *job,
   /* we know the total list of files to process */
   thunar_job_set_total_files (THUNAR_JOB (job), file_list);
 
+  /* take a reference on the thumbnail cache */
+  application = thunar_application_get ();
+  thumbnail_cache = thunar_application_get_thumbnail_cache (application);
+  g_object_unref (application);
+
   /* remove all the files */
   for (lp = file_list; lp != NULL && !exo_job_is_cancelled (EXO_JOB (job)); lp = lp->next)
     {
@@ -415,13 +456,19 @@ _thunar_io_jobs_unlink (ThunarJob   *job,
 
 again:
       /* try to delete the file */
-      if (!g_file_delete (lp->data, exo_job_get_cancellable (EXO_JOB (job)), &err))
+      if (g_file_delete (lp->data, exo_job_get_cancellable (EXO_JOB (job)), &err))
+        {
+          /* notify the thumbnail cache that the corresponding thumbnail can also
+           * be deleted now */
+          thunar_thumbnail_cache_delete_file (thumbnail_cache, lp->data);
+        }
+      else
         {
           /* query the file info for the display name */
-          info = g_file_query_info (lp->data, 
+          info = g_file_query_info (lp->data,
                                     G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
-                                    G_FILE_QUERY_INFO_NONE, 
-                                    exo_job_get_cancellable (EXO_JOB (job)), 
+                                    G_FILE_QUERY_INFO_NONE,
+                                    exo_job_get_cancellable (EXO_JOB (job)),
                                     NULL);
 
           /* abort if the job was cancelled */
@@ -445,8 +492,8 @@ again:
             }
 
           /* ask the user whether he wants to skip this file */
-          response = thunar_job_ask_skip (THUNAR_JOB (job), 
-                                          _("Could not delete file \"%s\": %s"), 
+          response = thunar_job_ask_skip (THUNAR_JOB (job),
+                                          _("Could not delete file \"%s\": %s"),
                                           display_name, err->message);
           g_free (display_name);
 
@@ -459,6 +506,9 @@ again:
         }
     }
 
+  /* release the thumbnail cache */
+  g_object_unref (thumbnail_cache);
+
   /* release the file list */
   thunar_g_file_list_free (file_list);
 
@@ -489,9 +539,9 @@ thunar_io_jobs_move_files (GList *source_file_list,
   _thunar_return_val_if_fail (target_file_list != NULL, NULL);
   _thunar_return_val_if_fail (g_list_length (source_file_list) == g_list_length (target_file_list), NULL);
 
-  job = thunar_transfer_job_new (source_file_list, target_file_list, 
+  job = thunar_transfer_job_new (source_file_list, target_file_list,
                                  THUNAR_TRANSFER_JOB_MOVE);
-  
+
   return THUNAR_JOB (exo_job_launch (EXO_JOB (job)));
 }
 
@@ -556,7 +606,7 @@ _thunar_io_jobs_link_file (ThunarJob *job,
       if (!g_file_equal (source_file, target_file))
         {
           /* try to create the symlink */
-          if (g_file_make_symbolic_link (target_file, source_path, 
+          if (g_file_make_symbolic_link (target_file, source_path,
                                          exo_job_get_cancellable (EXO_JOB (job)),
                                          &err))
             {
@@ -589,7 +639,7 @@ _thunar_io_jobs_link_file (ThunarJob *job,
                       /* return the real target file */
                       return duplicate_file;
                     }
-                  
+
                   /* release the duplicate file, we no longer need it */
                   g_object_unref (duplicate_file);
                 }
@@ -618,7 +668,7 @@ _thunar_io_jobs_link_file (ThunarJob *job,
           /* try to delete the file */
           if (response == THUNAR_JOB_RESPONSE_YES)
             {
-              /* try to remove the target file. if not possible, err will be set and 
+              /* try to remove the target file. if not possible, err will be set and
                * the while loop will be aborted */
               g_file_delete (target_file, exo_job_get_cancellable (EXO_JOB (job)), &err);
             }
@@ -642,29 +692,36 @@ _thunar_io_jobs_link_file (ThunarJob *job,
 
 
 static gboolean
-_thunar_io_jobs_link (ThunarJob   *job,
-                      GValueArray *param_values,
-                      GError     **error)
+_thunar_io_jobs_link (ThunarJob  *job,
+                      GArray     *param_values,
+                      GError    **error)
 {
-  GError *err = NULL;
-  GFile  *real_target_file;
-  GList  *new_files_list = NULL;
-  GList  *source_file_list;
-  GList  *sp;
-  GList  *target_file_list;
-  GList  *tp;
+  ThunarThumbnailCache *thumbnail_cache;
+  ThunarApplication    *application;
+  GError               *err = NULL;
+  GFile                *real_target_file;
+  GList                *new_files_list = NULL;
+  GList                *source_file_list;
+  GList                *sp;
+  GList                *target_file_list;
+  GList                *tp;
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 2, FALSE);
+  _thunar_return_val_if_fail (param_values->len == 2, FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  source_file_list = g_value_get_boxed (g_value_array_get_nth (param_values, 0));
-  target_file_list = g_value_get_boxed (g_value_array_get_nth (param_values, 1));
+  source_file_list = g_value_get_boxed (&g_array_index (param_values, GValue, 0));
+  target_file_list = g_value_get_boxed (&g_array_index (param_values, GValue, 1));
 
   /* we know the total list of paths to process */
   thunar_job_set_total_files (THUNAR_JOB (job), source_file_list);
 
+  /* take a reference on the thumbnail cache */
+  application = thunar_application_get ();
+  thumbnail_cache = thunar_application_get_thumbnail_cache (application);
+  g_object_unref (application);
+
   /* process all files */
   for (sp = source_file_list, tp = target_file_list;
        err == NULL && sp != NULL && tp != NULL;
@@ -683,15 +740,24 @@ _thunar_io_jobs_link (ThunarJob   *job,
           /* queue the file for the folder update unless it was skipped */
           if (sp->data != real_target_file)
             {
-              new_files_list = thunar_g_file_list_prepend (new_files_list, 
+              new_files_list = thunar_g_file_list_prepend (new_files_list,
                                                            real_target_file);
+
+              /* notify the thumbnail cache that we need to copy the original
+               * thumbnail for the symlink to have one too */
+              thunar_thumbnail_cache_copy_file (thumbnail_cache, sp->data,
+                                                real_target_file);
+
             }
-  
+
           /* release the real target file */
           g_object_unref (real_target_file);
         }
     }
 
+  /* release the thumbnail cache */
+  g_object_unref (thumbnail_cache);
+
   if (err != NULL)
     {
       thunar_g_file_list_free (new_files_list);
@@ -724,30 +790,45 @@ thunar_io_jobs_link_files (GList *source_file_list,
 
 
 static gboolean
-_thunar_io_jobs_trash (ThunarJob   *job,
-                       GValueArray *param_values,
-                       GError     **error)
+_thunar_io_jobs_trash (ThunarJob  *job,
+                       GArray     *param_values,
+                       GError    **error)
 {
-  GError *err = NULL;
-  GList  *file_list;
-  GList  *lp;
+  ThunarThumbnailCache *thumbnail_cache;
+  ThunarApplication    *application;
+  GError               *err = NULL;
+  GList                *file_list;
+  GList                *lp;
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 1, FALSE);
+  _thunar_return_val_if_fail (param_values->len == 1, FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  file_list = g_value_get_boxed (g_value_array_get_nth (param_values, 0));
+  file_list = g_value_get_boxed (&g_array_index (param_values, GValue, 0));
 
   if (exo_job_set_error_if_cancelled (EXO_JOB (job), error))
     return FALSE;
 
+  /* take a reference on the thumbnail cache */
+  application = thunar_application_get ();
+  thumbnail_cache = thunar_application_get_thumbnail_cache (application);
+  g_object_unref (application);
+
   for (lp = file_list; err == NULL && lp != NULL; lp = lp->next)
     {
       _thunar_assert (G_IS_FILE (lp->data));
+
+      /* trash the file or folder */
       g_file_trash (lp->data, exo_job_get_cancellable (EXO_JOB (job)), &err);
+
+      /* update the thumbnail cache */
+      thunar_thumbnail_cache_cleanup_file (thumbnail_cache, lp->data);
     }
 
+  /* release the thumbnail cache */
+  g_object_unref (thumbnail_cache);
+
   if (err != NULL)
     {
       g_propagate_error (error, err);
@@ -782,7 +863,7 @@ thunar_io_jobs_restore_files (GList *source_file_list,
   _thunar_return_val_if_fail (target_file_list != NULL, NULL);
   _thunar_return_val_if_fail (g_list_length (source_file_list) == g_list_length (target_file_list), NULL);
 
-  job = thunar_transfer_job_new (source_file_list, target_file_list, 
+  job = thunar_transfer_job_new (source_file_list, target_file_list,
                                  THUNAR_TRANSFER_JOB_MOVE);
 
   return THUNAR_JOB (exo_job_launch (EXO_JOB (job)));
@@ -791,9 +872,9 @@ thunar_io_jobs_restore_files (GList *source_file_list,
 
 
 static gboolean
-_thunar_io_jobs_chown (ThunarJob   *job,
-                       GValueArray *param_values,
-                       GError     **error)
+_thunar_io_jobs_chown (ThunarJob  *job,
+                       GArray     *param_values,
+                       GError    **error)
 {
   ThunarJobResponse response;
   const gchar      *message;
@@ -807,13 +888,13 @@ _thunar_io_jobs_chown (ThunarJob   *job,
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 4, FALSE);
+  _thunar_return_val_if_fail (param_values->len == 4, FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  file_list = g_value_get_boxed (g_value_array_get_nth (param_values, 0));
-  uid = g_value_get_int (g_value_array_get_nth (param_values, 1));
-  gid = g_value_get_int (g_value_array_get_nth (param_values, 2));
-  recursive = g_value_get_boolean (g_value_array_get_nth (param_values, 3));
+  file_list = g_value_get_boxed (&g_array_index (param_values, GValue, 0));
+  uid = g_value_get_int (&g_array_index (param_values, GValue, 1));
+  gid = g_value_get_int (&g_array_index (param_values, GValue, 2));
+  recursive = g_value_get_boolean (&g_array_index (param_values, GValue, 3));
 
   _thunar_assert ((uid >= 0 || gid >= 0) && !(uid >= 0 && gid >= 0));
 
@@ -839,7 +920,7 @@ _thunar_io_jobs_chown (ThunarJob   *job,
       thunar_job_processing_file (THUNAR_JOB (job), lp);
 
       /* try to query information about the file */
-      info = g_file_query_info (lp->data, 
+      info = g_file_query_info (lp->data,
                                 G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
                                 G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
                                 exo_job_get_cancellable (EXO_JOB (job)),
@@ -872,11 +953,11 @@ retry_chown:
       if (err != NULL && !exo_job_is_cancelled (EXO_JOB (job)))
         {
           /* generate a useful error message */
-          message = G_LIKELY (uid >= 0) ? _("Failed to change the owner of \"%s\": %s") 
+          message = G_LIKELY (uid >= 0) ? _("Failed to change the owner of \"%s\": %s")
                                         : _("Failed to change the group of \"%s\": %s");
 
           /* ask the user whether to skip/retry this file */
-          response = thunar_job_ask_skip (THUNAR_JOB (job), message, 
+          response = thunar_job_ask_skip (THUNAR_JOB (job), message,
                                           g_file_info_get_display_name (info),
                                           err->message);
 
@@ -909,33 +990,28 @@ retry_chown:
 
 
 ThunarJob *
-thunar_io_jobs_change_group (GFile    *file,
+thunar_io_jobs_change_group (GList    *files,
                              guint32   gid,
                              gboolean  recursive)
 {
-  GList file_list;
+  _thunar_return_val_if_fail (files != NULL, NULL);
 
-  _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
+  /* files are released when the list if destroyed */
+  g_list_foreach (files, (GFunc) g_object_ref, NULL);
 
-  file_list.data = g_object_ref (file);
-  file_list.next = NULL; 
-  file_list.prev = NULL;
-  
   return thunar_simple_job_launch (_thunar_io_jobs_chown, 4,
-                                   THUNAR_TYPE_G_FILE_LIST, &file_list,
+                                   THUNAR_TYPE_G_FILE_LIST, files,
                                    G_TYPE_INT, -1,
                                    G_TYPE_INT, (gint) gid,
                                    G_TYPE_BOOLEAN, recursive);
-
-  g_object_unref (file_list.data);
 }
 
 
 
 static gboolean
-_thunar_io_jobs_chmod (ThunarJob   *job,
-                       GValueArray *param_values,
-                       GError     **error)
+_thunar_io_jobs_chmod (ThunarJob  *job,
+                       GArray     *param_values,
+                       GError    **error)
 {
   ThunarJobResponse response;
   GFileInfo        *info;
@@ -954,15 +1030,15 @@ _thunar_io_jobs_chmod (ThunarJob   *job,
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 6, FALSE);
+  _thunar_return_val_if_fail (param_values->len == 6, FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  file_list = g_value_get_boxed (g_value_array_get_nth (param_values, 0));
-  dir_mask = g_value_get_flags (g_value_array_get_nth (param_values, 1));
-  dir_mode = g_value_get_flags (g_value_array_get_nth (param_values, 2));
-  file_mask = g_value_get_flags (g_value_array_get_nth (param_values, 3));
-  file_mode = g_value_get_flags (g_value_array_get_nth (param_values, 4));
-  recursive = g_value_get_boolean (g_value_array_get_nth (param_values, 5));
+  file_list = g_value_get_boxed (&g_array_index (param_values, GValue, 0));
+  dir_mask = g_value_get_flags (&g_array_index (param_values, GValue, 1));
+  dir_mode = g_value_get_flags (&g_array_index (param_values, GValue, 2));
+  file_mask = g_value_get_flags (&g_array_index (param_values, GValue, 3));
+  file_mode = g_value_get_flags (&g_array_index (param_values, GValue, 4));
+  recursive = g_value_get_boolean (&g_array_index (param_values, GValue, 5));
 
   /* collect the files for the chown operation */
   if (recursive)
@@ -986,7 +1062,7 @@ _thunar_io_jobs_chmod (ThunarJob   *job,
       thunar_job_processing_file (THUNAR_JOB (job), lp);
 
       /* try to query information about the file */
-      info = g_file_query_info (lp->data, 
+      info = g_file_query_info (lp->data,
                                 G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
                                 G_FILE_ATTRIBUTE_STANDARD_TYPE ","
                                 G_FILE_ATTRIBUTE_UNIX_MODE,
@@ -1013,23 +1089,26 @@ retry_chown:
       /* determine the current mode */
       old_mode = g_file_info_get_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_MODE);
 
-      /* generate the new mode, taking the old mode (which contains file type 
+      /* generate the new mode, taking the old mode (which contains file type
        * information) into account */
       new_mode = ((old_mode & ~mask) | mode) & 07777;
 
-      /* try to change the file mode */
-      g_file_set_attribute_uint32 (lp->data,
-                                   G_FILE_ATTRIBUTE_UNIX_MODE, new_mode,
-                                   G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-                                   exo_job_get_cancellable (EXO_JOB (job)),
-                                   &err);
+      if (old_mode != new_mode)
+        {
+          /* try to change the file mode */
+          g_file_set_attribute_uint32 (lp->data,
+                                       G_FILE_ATTRIBUTE_UNIX_MODE, new_mode,
+                                       G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                       exo_job_get_cancellable (EXO_JOB (job)),
+                                       &err);
+        }
 
       /* check if there was a recoverable error */
       if (err != NULL && !exo_job_is_cancelled (EXO_JOB (job)))
         {
           /* ask the user whether to skip/retry this file */
           response = thunar_job_ask_skip (job,
-                                          _("Failed to change the permissions of \"%s\": %s"), 
+                                          _("Failed to change the permissions of \"%s\": %s"),
                                           g_file_info_get_display_name (info),
                                           err->message);
 
@@ -1063,77 +1142,56 @@ retry_chown:
 
 
 ThunarJob *
-thunar_io_jobs_change_mode (GFile         *file,
+thunar_io_jobs_change_mode (GList         *files,
                             ThunarFileMode dir_mask,
                             ThunarFileMode dir_mode,
                             ThunarFileMode file_mask,
                             ThunarFileMode file_mode,
                             gboolean       recursive)
 {
-  GList file_list;
+  _thunar_return_val_if_fail (files != NULL, NULL);
 
-  _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
+  /* files are released when the list if destroyed */
+  g_list_foreach (files, (GFunc) g_object_ref, NULL);
 
-  file_list.data = g_object_ref (file);
-  file_list.next = NULL; 
-  file_list.prev = NULL;
-  
   return thunar_simple_job_launch (_thunar_io_jobs_chmod, 6,
-                                   THUNAR_TYPE_G_FILE_LIST, &file_list,
+                                   THUNAR_TYPE_G_FILE_LIST, files,
                                    THUNAR_TYPE_FILE_MODE, dir_mask,
                                    THUNAR_TYPE_FILE_MODE, dir_mode,
                                    THUNAR_TYPE_FILE_MODE, file_mask,
                                    THUNAR_TYPE_FILE_MODE, file_mode,
                                    G_TYPE_BOOLEAN, recursive);
-
-  g_object_unref (file_list.data);
 }
 
 
 
 static gboolean
-_thunar_io_jobs_ls (ThunarJob   *job,
-                    GValueArray *param_values,
-                    GError     **error)
+_thunar_io_jobs_ls (ThunarJob  *job,
+                    GArray     *param_values,
+                    GError    **error)
 {
-  ThunarFile *file;
-  GError     *err = NULL;
-  GFile      *directory;
-  GList      *file_list = NULL;
-  GList      *lp;
-  GList      *path_list;
+  GError *err = NULL;
+  GFile  *directory;
+  GList  *file_list = NULL;
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 1, FALSE);
+  _thunar_return_val_if_fail (param_values->len == 1, FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   if (exo_job_set_error_if_cancelled (EXO_JOB (job), error))
     return FALSE;
 
   /* determine the directory to list */
-  directory = g_value_get_object (g_value_array_get_nth (param_values, 0));
+  directory = g_value_get_object (&g_array_index (param_values, GValue, 0));
 
   /* make sure the object is valid */
   _thunar_assert (G_IS_FILE (directory));
 
   /* collect directory contents (non-recursively) */
-  path_list = thunar_io_scan_directory (job, directory, 
-                                        G_FILE_QUERY_INFO_NONE, 
-                                        FALSE, FALSE, &err);
-
-  /* turn the GFile list into a ThunarFile list */
-  for (lp = g_list_last (path_list); 
-       err == NULL && !exo_job_is_cancelled (EXO_JOB (job)) && lp != NULL; 
-       lp = lp->prev)
-    {
-      file = thunar_file_get (lp->data, &err);
-      if (G_LIKELY (file != NULL))
-        file_list = g_list_prepend (file_list, file);
-    }
-
-  /* free the GFile list */
-  thunar_g_file_list_free (path_list);
+  file_list = thunar_io_scan_directory (job, directory,
+                                        G_FILE_QUERY_INFO_NONE,
+                                        FALSE, FALSE, TRUE, &err);
 
   /* abort on errors or cancellation */
   if (err != NULL)
@@ -1155,10 +1213,10 @@ _thunar_io_jobs_ls (ThunarJob   *job,
         {
           /* none of the handlers took over the file list, so it's up to us
            * to destroy it */
-          thunar_file_list_free (file_list);
+          thunar_g_file_list_free (file_list);
         }
     }
-  
+
   /* there should be no errors here */
   _thunar_assert (err == NULL);
 
@@ -1178,7 +1236,7 @@ ThunarJob *
 thunar_io_jobs_list_directory (GFile *directory)
 {
   _thunar_return_val_if_fail (G_IS_FILE (directory), NULL);
-  
+
   return thunar_simple_job_launch (_thunar_io_jobs_ls, 1, G_TYPE_FILE, directory);
 }
 
@@ -1201,9 +1259,9 @@ _thunar_io_jobs_rename_notify (ThunarFile *file)
 
 
 static gboolean
-_thunar_io_jobs_rename (ThunarJob   *job,
-                        GValueArray *param_values,
-                        GError     **error)
+_thunar_io_jobs_rename (ThunarJob  *job,
+                        GArray     *param_values,
+                        GError    **error)
 {
   const gchar *display_name;
   ThunarFile  *file;
@@ -1211,23 +1269,23 @@ _thunar_io_jobs_rename (ThunarJob   *job,
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->n_values == 2, FALSE);
-  _thunar_return_val_if_fail (G_VALUE_HOLDS (&param_values->values[0], THUNAR_TYPE_FILE), FALSE);
-  _thunar_return_val_if_fail (G_VALUE_HOLDS_STRING (&param_values->values[1]), FALSE);
+  _thunar_return_val_if_fail (param_values->len == 2, FALSE);
+  _thunar_return_val_if_fail (G_VALUE_HOLDS (&g_array_index (param_values, GValue, 0), THUNAR_TYPE_FILE), FALSE);
+  _thunar_return_val_if_fail (G_VALUE_HOLDS_STRING (&g_array_index (param_values, GValue, 1)), FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   if (exo_job_set_error_if_cancelled (EXO_JOB (job), error))
     return FALSE;
 
   /* determine the file and display name */
-  file = g_value_get_object (g_value_array_get_nth (param_values, 0));
-  display_name = g_value_get_string (g_value_array_get_nth (param_values, 1));
+  file = g_value_get_object (&g_array_index (param_values, GValue, 0));
+  display_name = g_value_get_string (&g_array_index (param_values, GValue, 1));
 
   /* try to rename the file */
   if (thunar_file_rename (file, display_name, exo_job_get_cancellable (EXO_JOB (job)), TRUE, &err))
     {
-      exo_job_send_to_mainloop (EXO_JOB (job), 
-                                (GSourceFunc) _thunar_io_jobs_rename_notify, 
+      exo_job_send_to_mainloop (EXO_JOB (job),
+                                (GSourceFunc) _thunar_io_jobs_rename_notify,
                                 g_object_ref (file), g_object_unref);
     }
 
@@ -1250,7 +1308,7 @@ thunar_io_jobs_rename_file (ThunarFile  *file,
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), NULL);
   _thunar_return_val_if_fail (g_utf8_validate (display_name, -1, NULL), NULL);
 
-  return thunar_simple_job_launch (_thunar_io_jobs_rename, 2, 
-                                   THUNAR_TYPE_FILE, file, 
+  return thunar_simple_job_launch (_thunar_io_jobs_rename, 2,
+                                   THUNAR_TYPE_FILE, file,
                                    G_TYPE_STRING, display_name);
 }
