@@ -221,7 +221,7 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
       g_param_spec_string ("last-details-view-column-order",
                            "LastDetailsViewColumnOrder",
                            NULL,
-                           "THUNAR_COLUMN_NAME,THUNAR_COLUMN_SIZE,THUNAR_COLUMN_TYPE,THUNAR_COLUMN_DATE_MODIFIED",
+                           "THUNAR_COLUMN_NAME,THUNAR_COLUMN_SIZE,THUNAR_COLUMN_SIZE_IN_BYTES,THUNAR_COLUMN_TYPE,THUNAR_COLUMN_DATE_MODIFIED",
                            EXO_PARAM_READWRITE);
 
   /**
@@ -575,7 +575,7 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
   /**
    * ThunarPreferences:misc-recursive-permissions:
    *
-   * Whether to apply permissions recursively everytime the
+   * Whether to apply permissions recursively every time the
    * permissions are altered by the user.
    **/
   preferences_props[PROP_MISC_RECURSIVE_PERMISSIONS] =
