@@ -37,6 +37,7 @@
 #include <thunar/thunar-icon-factory.h>
 #include <thunar/thunar-icon-renderer.h>
 #include <thunar/thunar-launcher.h>
+#include <thunar/thunar-menu-util.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-properties-dialog.h>
 #include <thunar/thunar-renamer-dialog.h>
@@ -78,9 +79,7 @@ static void        thunar_renamer_dialog_realize               (GtkWidget
 static void        thunar_renamer_dialog_unrealize             (GtkWidget                *widget);
 static void        thunar_renamer_dialog_response              (GtkDialog                *dialog,
                                                                 gint                      response);
-static void        thunar_renamer_dialog_context_menu          (ThunarRenamerDialog      *renamer_dialog,
-                                                                guint                     button,
-                                                                guint32                   timestamp);
+static void        thunar_renamer_dialog_context_menu          (ThunarRenamerDialog      *renamer_dialog);
 static void        thunar_renamer_dialog_help                  (ThunarRenamerDialog      *renamer_dialog);
 static void        thunar_renamer_dialog_save                  (ThunarRenamerDialog      *renamer_dialog);
 static void        thunar_renamer_dialog_action_add_files      (GtkAction                *action,
@@ -190,11 +189,11 @@ static const GtkActionEntry action_entries[] =
   { "file-menu", NULL, N_ ("_File"), NULL, },
   { "sendto-menu", NULL, N_ ("_Send To"), NULL, },
   { "file-context-menu", NULL, N_ ("File Context Menu"), NULL, },
-  { "add-files", GTK_STOCK_ADD, N_ ("_Add Files..."), NULL, N_ ("Include additional files in the list of files to be renamed"), G_CALLBACK (thunar_renamer_dialog_action_add_files), },
-  { "remove-files", GTK_STOCK_REMOVE, NULL, NULL, NULL, G_CALLBACK (thunar_renamer_dialog_action_remove_files), },
-  { "clear", GTK_STOCK_CLEAR, N_ ("Clear"), NULL, N_ ("Clear the file list below"), G_CALLBACK (thunar_renamer_dialog_action_clear), },
-  { "about", GTK_STOCK_ABOUT, N_ ("_About"), NULL, N_ ("Display information about Thunar Bulk Rename"), G_CALLBACK (thunar_renamer_dialog_action_about), },
-  { "properties", GTK_STOCK_PROPERTIES, N_ ("_Properties..."), "<alt>Return", N_ ("View the properties of the selected file"), G_CALLBACK (thunar_renamer_dialog_action_properties), },
+  { "add-files", "list-add", N_ ("_Add Files..."), NULL, N_ ("Include additional files in the list of files to be renamed"), G_CALLBACK (thunar_renamer_dialog_action_add_files), },
+  { "remove-files", "list-remove", NULL, NULL, NULL, G_CALLBACK (thunar_renamer_dialog_action_remove_files), },
+  { "clear", "edit-clear", N_ ("Clear"), NULL, N_ ("Clear the file list below"), G_CALLBACK (thunar_renamer_dialog_action_clear), },
+  { "about", "help-about", N_ ("_About"), NULL, N_ ("Display information about Thunar Bulk Rename"), G_CALLBACK (thunar_renamer_dialog_action_about), },
+  { "properties", "document-properties", N_ ("_Properties..."), "<alt>Return", N_ ("View the properties of the selected file"), G_CALLBACK (thunar_renamer_dialog_action_properties), },
 };
 
 /* Target types for dropping to the tree view */
@@ -352,8 +351,8 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   gtk_window_set_title (GTK_WINDOW (renamer_dialog), _("Rename Multiple Files"));
 
   /* add the Cancel/Close buttons */
-  renamer_dialog->cancel_button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-  renamer_dialog->close_button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT);
+  renamer_dialog->cancel_button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), _("_Cancel"), GTK_RESPONSE_CANCEL);
+  renamer_dialog->close_button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), _("_Close"), GTK_RESPONSE_DELETE_EVENT);
   gtk_widget_hide (renamer_dialog->close_button);
 
   /* add the "Rename Files" button */
@@ -362,6 +361,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   gtk_dialog_set_default_response (GTK_DIALOG (renamer_dialog), GTK_RESPONSE_ACCEPT);
   gtk_widget_set_tooltip_text (button, _("Click here to actually rename the files listed above to their new names."));
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   /* setup the action group for this dialog */
   renamer_dialog->action_group = gtk_action_group_new ("ThunarRenamerDialog");
   gtk_action_group_set_translation_domain (renamer_dialog->action_group, GETTEXT_PACKAGE);
@@ -375,6 +375,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   /* connect the accelerators */
   accel_group = gtk_ui_manager_get_accel_group (renamer_dialog->ui_manager);
   gtk_window_add_accel_group (GTK_WINDOW (renamer_dialog), accel_group);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
   /* setup the launcher support for this dialog */
   renamer_dialog->launcher = thunar_launcher_new ();
@@ -383,14 +384,16 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   exo_binding_new (G_OBJECT (renamer_dialog), "selected-files", G_OBJECT (renamer_dialog->launcher), "selected-files");
 
   /* add the toolbar to the dialog */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   toolbar = gtk_ui_manager_get_widget (renamer_dialog->ui_manager, "/toolbar");
+G_GNUC_END_IGNORE_DEPRECATIONS
   exo_binding_new (G_OBJECT (renamer_dialog), "standalone", G_OBJECT (toolbar), "visible");
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (renamer_dialog)->vbox), toolbar, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (renamer_dialog))), toolbar, FALSE, FALSE, 0);
 
   /* create the main vbox */
-  mbox = gtk_vbox_new (FALSE, 12);
+  mbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (mbox), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (renamer_dialog)->vbox), mbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (renamer_dialog))), mbox, TRUE, TRUE, 0);
   gtk_widget_show (mbox);
 
   /* create the scrolled window for the tree view */
@@ -406,7 +409,6 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW (renamer_dialog->tree_view), TRUE);
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW (renamer_dialog->tree_view), TRUE);
   gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW (renamer_dialog->tree_view), TRUE);
-  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (renamer_dialog->tree_view), TRUE);
   g_signal_connect (G_OBJECT (renamer_dialog->tree_view), "button-press-event", G_CALLBACK (thunar_renamer_dialog_button_press_event), renamer_dialog);
   g_signal_connect (G_OBJECT (renamer_dialog->tree_view), "popup-menu", G_CALLBACK (thunar_renamer_dialog_popup_menu), renamer_dialog);
   g_signal_connect (G_OBJECT (renamer_dialog->tree_view), "row-activated", G_CALLBACK (thunar_renamer_dialog_row_activated), renamer_dialog);
@@ -450,12 +452,12 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   gtk_tree_view_append_column (GTK_TREE_VIEW (renamer_dialog->tree_view), column);
 
   /* create the vbox for the renamer parameters */
-  vbox = gtk_vbox_new (FALSE, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_box_pack_start (GTK_BOX (mbox), vbox, FALSE, FALSE, 0);
   gtk_widget_show (vbox);
 
   /* create the hbox for the renamer selection */
-  rbox = gtk_hbox_new (FALSE, 3);
+  rbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
   gtk_box_pack_start (GTK_BOX (vbox), rbox, FALSE, FALSE, 0);
   gtk_widget_show (rbox);
 
@@ -503,7 +505,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
       gtk_box_pack_start (GTK_BOX (rbox), button, FALSE, FALSE, 0);
       gtk_widget_show (button);
 
-      image = gtk_image_new_from_stock (GTK_STOCK_HELP, GTK_ICON_SIZE_BUTTON);
+      image = gtk_image_new_from_icon_name ("help-browser", GTK_ICON_SIZE_BUTTON);
       gtk_container_add (GTK_CONTAINER (button), image);
       gtk_widget_show (image);
 
@@ -596,12 +598,12 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
       gtk_widget_set_sensitive (swin, FALSE);
 
       /* display an error to the user */
-      hbox = gtk_hbox_new (FALSE, 12);
+      hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
       gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
       gtk_container_add (GTK_CONTAINER (frame), hbox);
       gtk_widget_show (hbox);
 
-      image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_DIALOG);
+      image = gtk_image_new_from_icon_name ("dialog-error", GTK_ICON_SIZE_DIALOG);
       gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
       gtk_widget_show (image);
 
@@ -611,7 +613,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
       label = gtk_label_new (_("No renamer modules were found on your system. Please check your\n"
                                "installation or contact your system administrator. If you install Thunar\n"
                                "from source, be sure to enable the \"Simple Builtin Renamers\" plugin."));
-      gtk_misc_set_alignment (GTK_MISC (label), 0.0f, 0.5f);
+      gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
       gtk_label_set_selectable (GTK_LABEL (label), TRUE);
       gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
       gtk_widget_show (label);
@@ -840,15 +842,12 @@ thunar_renamer_dialog_response (GtkDialog *dialog,
 
 
 static void
-thunar_renamer_dialog_context_menu (ThunarRenamerDialog *renamer_dialog,
-                                    guint                button,
-                                    guint32              timestamp)
+thunar_renamer_dialog_context_menu (ThunarRenamerDialog *renamer_dialog)
 {
   GtkActionGroup *renamer_actions = NULL;
   ThunarxRenamer *renamer;
   GtkWidget      *menu;
-  GList          *actions = NULL;
-  GList          *lp;
+  GList          *items = NULL;
   gint            renamer_merge_id = 0;
 
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
@@ -860,45 +859,33 @@ thunar_renamer_dialog_context_menu (ThunarRenamerDialog *renamer_dialog,
   renamer = thunar_renamer_model_get_renamer (renamer_dialog->model);
   if (G_LIKELY (renamer != NULL))
     {
-      /* determine the actions provided by the active renamer */
-      actions = thunarx_renamer_get_actions (renamer, GTK_WINDOW (renamer_dialog), renamer_dialog->selected_files);
+      /* determine the menu items provided by the active renamer */
+      items = thunarx_renamer_get_menu_items (renamer, GTK_WINDOW (renamer_dialog), renamer_dialog->selected_files);
     }
 
-  /* check if we have any renamer actions */
-  if (G_UNLIKELY (actions != NULL))
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  /* check if we have any renamer menu items */
+  if (G_UNLIKELY (items != NULL))
     {
-      /* allocate a new action group and the merge id for the custom actions */
+      /* allocate a new action group and the merge id for the custom items */
       renamer_actions = gtk_action_group_new ("thunar-renamer-dialog-renamer-actions");
       renamer_merge_id = gtk_ui_manager_new_merge_id (renamer_dialog->ui_manager);
       gtk_ui_manager_insert_action_group (renamer_dialog->ui_manager, renamer_actions, -1);
 
-      /* add the actions to the UI manager */
-      for (lp = actions; lp != NULL; lp = lp->next)
-        {
-          /* add the action to the action group */
-          gtk_action_group_add_action (renamer_actions, GTK_ACTION (lp->data));
-
-          /* add the action to the UI manager */
-          gtk_ui_manager_add_ui (renamer_dialog->ui_manager, renamer_merge_id,
-                                 "/file-context-menu/placeholder-renamer-actions",
-                                 gtk_action_get_name (GTK_ACTION (lp->data)),
-                                 gtk_action_get_name (GTK_ACTION (lp->data)),
-                                 GTK_UI_MANAGER_MENUITEM, FALSE);
-
-          /* release the reference on the action */
-          g_object_unref (G_OBJECT (lp->data));
-        }
+      /* add the items to the UI manager */
+      thunar_menu_util_add_items_to_ui_manager (renamer_dialog->ui_manager, renamer_actions, renamer_merge_id,
+                                                "/file-context-menu/placeholder-renamer-actions", items);
 
       /* be sure to update the UI manager to avoid flickering */
       gtk_ui_manager_ensure_update (renamer_dialog->ui_manager);
 
       /* cleanup */
-      g_list_free (actions);
+      g_list_free (items);
     }
 
-  /* run the menu on the dialog's screen */
+  /* run the menu (takes over the floating of menu) */
   menu = gtk_ui_manager_get_widget (renamer_dialog->ui_manager, "/file-context-menu");
-  thunar_gtk_menu_run (GTK_MENU (menu), GTK_WIDGET (renamer_dialog), NULL, NULL, button, timestamp);
+  thunar_gtk_menu_run (GTK_MENU (menu));
 
   /* remove the previously merge items from the UI manager */
   if (G_UNLIKELY (renamer_merge_id != 0))
@@ -914,6 +901,7 @@ thunar_renamer_dialog_context_menu (ThunarRenamerDialog *renamer_dialog,
       g_object_unref (G_OBJECT (renamer_actions));
     }
 
+G_GNUC_END_IGNORE_DEPRECATIONS
   /* release the additional reference on the dialog */
   g_object_unref (G_OBJECT (renamer_dialog));
 }
@@ -925,8 +913,9 @@ thunar_renamer_dialog_help (ThunarRenamerDialog *renamer_dialog)
 {
   ThunarxRenamer *renamer;
   const gchar    *help_url = NULL;
-  GdkScreen      *screen;
   GError         *error = NULL;
+  gboolean        uri_launched = FALSE;
+
 
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
 
@@ -944,11 +933,13 @@ thunar_renamer_dialog_help (ThunarRenamerDialog *renamer_dialog)
     }
   else
     {
-      /* determine the dialog's screen */
-      screen = gtk_widget_get_screen (GTK_WIDGET (renamer_dialog));
-
       /* try to launch the specific URL */
-      if (!gtk_show_uri (screen, help_url, gtk_get_current_event_time (), &error))
+      uri_launched = gtk_show_uri_on_window (GTK_WINDOW (renamer_dialog),
+                                             help_url,
+                                             gtk_get_current_event_time (),
+                                             &error);
+
+      if (!uri_launched)
         {
           /* tell the user that we failed */
           thunar_dialogs_show_error (GTK_WIDGET (renamer_dialog), error, _("Failed to open the documentation browser"));
@@ -1050,15 +1041,17 @@ thunar_renamer_dialog_action_add_files (GtkAction           *action,
   GSList        *lp;
   gchar         *uri;
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (GTK_IS_ACTION (action));
+G_GNUC_END_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
 
   /* allocate the file chooser */
   chooser = gtk_file_chooser_dialog_new (_("Select files to rename"),
                                          GTK_WINDOW (renamer_dialog),
                                          GTK_FILE_CHOOSER_ACTION_OPEN,
-                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                         _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                         _("_Open"), GTK_RESPONSE_ACCEPT,
                                          NULL);
   gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), TRUE);
   gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (chooser), TRUE);
@@ -1156,7 +1149,9 @@ thunar_renamer_dialog_action_remove_files (GtkAction           *action,
   GList               *rows;
   GList               *lp;
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (GTK_IS_ACTION (action));
+G_GNUC_END_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
 
   /* determine the selected rows in the view */
@@ -1196,7 +1191,9 @@ static void
 thunar_renamer_dialog_action_clear (GtkAction           *action,
                                     ThunarRenamerDialog *renamer_dialog)
 {
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (GTK_IS_ACTION (action));
+G_GNUC_END_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
 
   /* just clear the list of files in the model */
@@ -1209,7 +1206,9 @@ static void
 thunar_renamer_dialog_action_about (GtkAction           *action,
                                     ThunarRenamerDialog *renamer_dialog)
 {
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (GTK_IS_ACTION (action));
+G_GNUC_END_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
 
   /* just popup the about dialog */
@@ -1226,7 +1225,9 @@ thunar_renamer_dialog_action_properties (GtkAction           *action,
 {
   GtkWidget *dialog;
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (GTK_IS_ACTION (action));
+G_GNUC_END_IGNORE_DEPRECATIONS
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
 
   /* popup the properties dialog */
@@ -1313,7 +1314,7 @@ thunar_renamer_dialog_button_press_event (GtkWidget           *tree_view,
         }
 
       /* popup the context menu */
-      thunar_renamer_dialog_context_menu (renamer_dialog, event->button, event->time);
+      thunar_renamer_dialog_context_menu (renamer_dialog);
 
       /* we handled the event */
       return TRUE;
@@ -1346,7 +1347,7 @@ thunar_renamer_dialog_drag_data_received (GtkWidget           *tree_view,
   _thunar_return_if_fail (GTK_IS_TREE_VIEW (tree_view));
 
   /* we only accept text/uri-list drops with format 8 and atleast one byte of data */
-  if (info == TARGET_TEXT_URI_LIST && selection_data->format == 8 && selection_data->length > 0)
+  if (info == TARGET_TEXT_URI_LIST && gtk_selection_data_get_format (selection_data) == 8 && gtk_selection_data_get_length (selection_data) > 0)
     {
       /* determine the renamer model */
       model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
@@ -1368,7 +1369,7 @@ thunar_renamer_dialog_drag_data_received (GtkWidget           *tree_view,
         }
 
       /* determine the file list from the selection_data */
-      file_list = thunar_g_file_list_new_from_string ((const gchar *) selection_data->data);
+      file_list = thunar_g_file_list_new_from_string ((const gchar *) gtk_selection_data_get_data (selection_data));
 
       /* add all paths to the model */
       for (lp = file_list; lp != NULL; lp = lp->next)
@@ -1423,7 +1424,7 @@ thunar_renamer_dialog_drag_leave (GtkWidget           *tree_view,
       /* we use the tree view parent (the scrolled window),
        * as the drag_highlight doesn't work for tree views.
        */
-      gtk_drag_unhighlight (tree_view->parent);
+      gtk_drag_unhighlight (gtk_widget_get_parent (tree_view));
       renamer_dialog->drag_highlighted = FALSE;
     }
 }
@@ -1467,14 +1468,14 @@ thunar_renamer_dialog_drag_motion (GtkWidget           *tree_view,
           /* we use the tree view parent (the scrolled window),
            * as the drag_highlight doesn't work for tree views.
            */
-          gtk_drag_unhighlight (tree_view->parent);
+          gtk_drag_unhighlight (gtk_widget_get_parent (tree_view));
           renamer_dialog->drag_highlighted = FALSE;
         }
 
       /* we cannot handle the drop */
       return FALSE;
     }
-      
+
   /* compute the drop position */
   if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (tree_view), x, y, &path, &position))
     {
@@ -1507,7 +1508,7 @@ thunar_renamer_dialog_drag_motion (GtkWidget           *tree_view,
   else if (!renamer_dialog->drag_highlighted)
     {
       /* highlight the parent */
-      gtk_drag_highlight (tree_view->parent);
+      gtk_drag_highlight (gtk_widget_get_parent (tree_view));
       renamer_dialog->drag_highlighted = TRUE;
     }
 
@@ -1533,10 +1534,10 @@ thunar_renamer_dialog_drag_drop (GtkWidget           *tree_view,
   GtkTreeViewDropPosition  drop_pos;
   gint                     position = -1;
   GList                   *rows;
-  
+
   _thunar_return_val_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog), FALSE);
   _thunar_return_val_if_fail (GTK_IS_TREE_VIEW (tree_view), FALSE);
-  
+
   /* determine the drop target */
   target = gtk_drag_dest_find_target (tree_view, context, NULL);
   if (G_LIKELY (target == gdk_atom_intern_static_string ("text/uri-list")))
@@ -1584,7 +1585,7 @@ thunar_renamer_dialog_drag_drop (GtkWidget           *tree_view,
 
   return TRUE;
 }
-                                  
+
 
 
 static void
@@ -1620,7 +1621,7 @@ thunar_renamer_dialog_popup_menu (GtkWidget           *tree_view,
   _thunar_return_val_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog), FALSE);
 
   /* popup the context menu */
-  thunar_renamer_dialog_context_menu (renamer_dialog, 0, gtk_get_current_event_time ());
+  thunar_renamer_dialog_context_menu (renamer_dialog);
 
   /* we handled the event */
   return TRUE;
@@ -1641,8 +1642,10 @@ thunar_renamer_dialog_row_activated (GtkTreeView         *tree_view,
 
   /* just activate the "open" action */
   action = thunar_gtk_ui_manager_get_action_by_name (renamer_dialog->ui_manager, "open");
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   if (G_LIKELY (action != NULL))
     gtk_action_activate (action);
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 
@@ -1684,6 +1687,7 @@ thunar_renamer_dialog_selection_changed (GtkTreeSelection    *selection,
     }
   g_list_free (rows);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   /* the "Remove Files" action is only sensitive if we have one or more files in the selection */
   action = gtk_action_group_get_action (renamer_dialog->action_group, "remove-files");
   g_object_set (G_OBJECT (action),
@@ -1697,6 +1701,7 @@ thunar_renamer_dialog_selection_changed (GtkTreeSelection    *selection,
   /* the "Properties" action is only sensitive if we have exactly one selected file */
   action = gtk_action_group_get_action (renamer_dialog->action_group, "properties");
   gtk_action_set_sensitive (action, n_selected_files > 0);
+  G_GNUC_END_IGNORE_DEPRECATIONS
 
   /* notify listeners */
   g_object_notify (G_OBJECT (renamer_dialog), "selected-files");
