@@ -1296,7 +1296,7 @@ thunar_shortcuts_model_load (gpointer data)
 
   _thunar_return_val_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model), FALSE);
 
-  GDK_THREADS_ENTER ();
+THUNAR_THREADS_ENTER
 
   /* parse the bookmarks */
   thunar_util_load_bookmarks (model->bookmarks_file,
@@ -1306,7 +1306,7 @@ thunar_shortcuts_model_load (gpointer data)
   /* update the visibility */
   thunar_shortcuts_model_header_visibility (model);
 
-  GDK_THREADS_LEAVE ();
+THUNAR_THREADS_LEAVE
 
   model->bookmarks_idle_id = 0;
 
@@ -1326,7 +1326,7 @@ thunar_shortcuts_model_reload (gpointer data)
 
   _thunar_return_val_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model), FALSE);
 
-  GDK_THREADS_ENTER ();
+THUNAR_THREADS_ENTER
 
   /* drop all existing user-defined shortcuts from the model */
   for (idx = 0, lp = model->shortcuts; lp != NULL; )
@@ -1357,7 +1357,7 @@ thunar_shortcuts_model_reload (gpointer data)
         }
     }
 
-  GDK_THREADS_LEAVE ();
+THUNAR_THREADS_LEAVE
 
   /* load new bookmarks */
   return thunar_shortcuts_model_load (data);
@@ -1955,7 +1955,7 @@ thunar_shortcuts_model_add (ThunarShortcutsModel *model,
     }
   else
     {
-      shortcut->location = g_object_ref (G_OBJECT (location));
+      shortcut->location = G_FILE (g_object_ref (G_OBJECT (location)));
       shortcut->gicon = g_themed_icon_new ("folder-remote");
     }
 
