@@ -99,7 +99,7 @@ struct _ThunarSizeLabel
 
 
 
-G_DEFINE_TYPE (ThunarSizeLabel, thunar_size_label, GTK_TYPE_HBOX)
+G_DEFINE_TYPE (ThunarSizeLabel, thunar_size_label, GTK_TYPE_BOX)
 
 
 
@@ -146,13 +146,14 @@ thunar_size_label_init (ThunarSizeLabel *size_label)
 {
   GtkWidget *ebox;
 
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (size_label), GTK_ORIENTATION_HORIZONTAL);
+
   /* binary file size */
   size_label->preferences = thunar_preferences_get ();
   exo_binding_new (G_OBJECT (size_label->preferences), "misc-file-size-binary",
                    G_OBJECT (size_label), "file-size-binary");
   g_signal_connect_swapped (G_OBJECT (size_label->preferences), "notify::misc-file-size-binary",
                             G_CALLBACK (thunar_size_label_files_changed), size_label);
-  gtk_widget_push_composite_child ();
 
   /* configure the box */
   gtk_box_set_spacing (GTK_BOX (size_label), 6);
@@ -172,13 +173,11 @@ thunar_size_label_init (ThunarSizeLabel *size_label)
 
   /* add the label widget */
   size_label->label = gtk_label_new (_("Calculating..."));
-  gtk_misc_set_alignment (GTK_MISC (size_label->label), 0.0f, 0.5f);
+  gtk_label_set_xalign (GTK_LABEL (size_label->label), 0.0f);
   gtk_label_set_selectable (GTK_LABEL (size_label->label), TRUE);
-  gtk_label_set_ellipsize (GTK_LABEL (size_label->label), PANGO_ELLIPSIZE_MIDDLE);
+  gtk_label_set_line_wrap (GTK_LABEL (size_label->label), TRUE);
   gtk_box_pack_start (GTK_BOX (size_label), size_label->label, TRUE, TRUE, 0);
   gtk_widget_show (size_label->label);
-
-  gtk_widget_pop_composite_child ();
 }
 
 
@@ -409,10 +408,10 @@ thunar_size_label_status_update (ThunarDeepCountJob *job,
   if (G_LIKELY (n > unreadable_directory_count))
     {
       /* update the label */
-      size_string = g_format_size_full (total_size, size_label->file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
+      size_string = g_format_size_full (total_size, G_FORMAT_SIZE_LONG_FORMAT | (size_label->file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT));
       text = g_strdup_printf (ngettext ("%u item, totalling %s", "%u items, totalling %s", n), n, size_string);
       g_free (size_string);
-      
+
       if (unreadable_directory_count > 0)
         {
           /* TRANSLATORS: this is shows if during the deep count size
@@ -421,7 +420,7 @@ thunar_size_label_status_update (ThunarDeepCountJob *job,
           g_free (text);
           text = unreable_text;
         }
-      
+
       gtk_label_set_text (GTK_LABEL (size_label->label), text);
       g_free (text);
     }
