@@ -37,6 +37,7 @@
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-tree-model.h>
 #include <thunar/thunar-device-monitor.h>
+#include <thunar/thunar-util.h>
 
 
 
@@ -918,13 +919,13 @@ thunar_tree_model_cleanup_idle (gpointer user_data)
 {
   ThunarTreeModel *model = THUNAR_TREE_MODEL (user_data);
 
-  GDK_THREADS_ENTER ();
+THUNAR_THREADS_ENTER
 
   /* walk through the tree and release all the nodes with a ref count of 0 */
   g_node_traverse (model->root, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
                    thunar_tree_model_node_traverse_cleanup, model);
 
-  GDK_THREADS_LEAVE ();
+THUNAR_THREADS_LEAVE
 
   return FALSE;
 }
@@ -1129,7 +1130,7 @@ thunar_tree_model_item_new_with_file (ThunarTreeModel *model,
   ThunarTreeModelItem *item;
 
   item = g_slice_new0 (ThunarTreeModelItem);
-  item->file = g_object_ref (G_OBJECT (file));
+  item->file = THUNAR_FILE (g_object_ref (G_OBJECT (file)));
   item->model = model;
 
   return item;
@@ -1145,7 +1146,7 @@ thunar_tree_model_item_new_with_device (ThunarTreeModel *model,
   GFile               *mount_point;
 
   item = g_slice_new0 (ThunarTreeModelItem);
-  item->device = g_object_ref (G_OBJECT (device));
+  item->device = THUNAR_DEVICE (g_object_ref (G_OBJECT (device)));
   item->model = model;
 
   /* check if the volume is mounted */
@@ -1434,7 +1435,7 @@ thunar_tree_model_item_load_idle (gpointer user_data)
       _thunar_return_val_if_fail (node->children == NULL || G_NODE_HAS_DUMMY (node), FALSE);
 #endif
 
-  GDK_THREADS_ENTER ();
+THUNAR_THREADS_ENTER
 
   /* check if we don't have a file yet and this is a mounted volume */
   if (item->file == NULL && item->device != NULL && thunar_device_is_mounted (item->device))
@@ -1471,7 +1472,7 @@ thunar_tree_model_item_load_idle (gpointer user_data)
         }
     }
 
-  GDK_THREADS_LEAVE ();
+THUNAR_THREADS_LEAVE
 
   return FALSE;
 }
